@@ -13,7 +13,7 @@
 #'
 #'
 #' @param dataSrc a connection to a dexter database, a matrix, or a data.frame with columns: person_id, item_id, item_score
-#' @param parms An object returned by function \code{fit_enorm} containing parameter estimates. If parms are provided, item parameters are condidered known. 
+#' @param parms An object returned by function \code{fit_enorm} containing parameter estimates. If parms are provided, item parameters are considered known. 
 #' If parms = NULL, plausible values are marginalized over the posterior distribution of the item parameters and uncertainty of the item parameters is taken into account.
 #' @param predicate an expression to filter data. If missing, the function will use 
 #' all data in dataSrc
@@ -21,7 +21,7 @@
 #' A covariate must be a discrete person property (e.g. not a float) that indicates nominal categories, e.g. gender or school.
 #' If dataSrc is a data.frame, it must contain the covariate.
 #' @param nPV Number of plausible values to draw per person.
-#' @param use_draw When the ENORM was fitted with a Gibbs sampler, this specifies the use of a particular sample of item parameters used to generate the plausibe value(s). If NULL, the posterior means are used. If outside range, the last iteration will be used.
+#' @param use_draw When the ENORM was fitted with a Gibbs sampler, this specifies the use of a particular sample of item parameters used to generate the plausible value(s). If NULL, the posterior means are used. If outside range, the last iteration will be used.
 #' @param prior.dist use a normal prior or a mixture of two normals
 #' recognised automatically), 
 #' @param merge_within_persons If a person took multiple booklets, this indicates
@@ -89,46 +89,69 @@ plausible_values_ = function(dataSrc, parms=NULL, qtpredicate=NULL, covariates=N
                              merge_within_persons=merge_within_persons)
 {
   if(is.null(env)) env = caller_env()
-  from = Gibbs.settings$from.pv ; step = Gibbs.settings$step.pv # burnin and thinning for pvs
+  from = Gibbs.settings$from.pv
+  step = Gibbs.settings$step.pv # burnin and thinning for pvs
   nIter.enorm = from + step*(nPV-1) # nr. of posterior samples of item parameters needed
+
+  
   prior.dist = match.arg(prior.dist)
 
-  if (!is.null(parms))
-  {
-    respData = get_resp_data(dataSrc, qtpredicate, summarised=TRUE, 
-                             extra_columns=covariates,env=env, 
-                             parms_check=parms$inputs$ssIS[,c('item_id','item_score')],
-                             merge_within_persons=FALSE)
-  } else
+  if(show_progress())
+    pg_start()
+  
+  if(is.null(parms))
   {
     respData = get_resp_data(dataSrc, qtpredicate, summarised=FALSE, extra_columns=covariates, env=env)
     parms = fit_enorm_(respData, method = 'Bayes', nIterations = nIter.enorm) 
     respData = get_resp_data(respData, summarised=TRUE, extra_columns=covariates, 
-                             protect_x=!inherits(dataSrc,'DBIConnection'))
-  } 
-
+                             protect_x=!is_db(dataSrc))
+  } else
+  {
+    if(inherits(parms,'data.frame'))
+    {
+      parms = transform.df.parms(parms,'b', TRUE)
+      pcheck = parms[,c('item_id','item_score')]
+    } else
+    {
+      pcheck = parms$inputs$ssIS[,c('item_id','item_score')]
+    }
+    
+    respData = get_resp_data(dataSrc, qtpredicate, summarised=TRUE, 
+                             extra_columns=covariates,env=env, 
+                             parms_check=pcheck,
+                             merge_within_persons=FALSE)
+  }
+  # to do: use the new function
+  if(inherits(parms,'data.frame'))
+  {
+    fl = parms %>%
+      mutate(rn=row_number()) %>%
+      group_by(.data$item_id) %>%
+      summarize(first=as.integer(min(.data$rn)), last=as.integer(max(.data$rn))) %>%
+      ungroup()
+    a = parms$item_score
+    b = parms$b
+  } else
+  {
+    fl = parms$inputs$ssI[c('item_id','first','last')]
+    a = parms$inputs$ssIS$item_score
+    b = parms$est$b
+    if(parms$inputs$method == 'Bayes' )
+    {
+      if(!is.null(use_draw))
+        b = b[use_draw,]
+      else if (nrow(b)<nIter.enorm ) 
+        stop(paste("To produce", nPV, "plausible values, use at least", nIter.enorm, "iterations in fit_enorm" ))
+    }  
+  }
   x = respData$x
   
   # join design with the params
   # these can have different levels
   design = suppressWarnings(respData$design %>%
-    left_join(parms$inputs$ssI, by='item_id') %>% 
+    inner_join(fl, by='item_id') %>% 
     arrange(.data$booklet_id, .data$first))
-  
-  if(any(is.na(design$first))) stop('Some of your items have no parameters')
-  
-  b = parms$est$b
-  a = parms$inputs$ssIS$item_score
-  
-  if(parms$inputs$method == 'Bayes' )
-  {
-    if(!is.null(use_draw))
-     b = b[use_draw,]
-    else if (nrow(b)<nIter.enorm ) 
-      stop(paste("To produce", nPV, "plausible values, use at least", nIter.enorm, "iterations in fit_enorm" ))
-  }  
 
-  
   if(!is.null(covariates))
   {
     group_number = (function(){i = 0L; function() i <<- i+1L })()
@@ -142,7 +165,7 @@ plausible_values_ = function(dataSrc, parms=NULL, qtpredicate=NULL, covariates=N
     # niet varierende pop toevoegen maakt code in pv eenvoudiger
     x$pop = 1L
   }
-  design = split(design, design$booklet_id)
+  design = split(design, design$booklet_id, drop=TRUE)
   
   y = pv(x[,c('booklet_id','person_id','booklet_score','pop')], 
          design, b, a, nPV, from = from, by = step, prior.dist=prior.dist)
