@@ -60,6 +60,8 @@ std::string ppoint(SEXP x)
 	return tfm::format("%p", x);
 }
 
+
+
 // [[Rcpp::export]]
 IntegerVector ds_connected_groups(const IntegerMatrix& a)
 {
@@ -92,6 +94,39 @@ IntegerVector ds_connected_groups(const IntegerMatrix& a)
 	return group;
 }
 
+// needs two groups with values 1 and 2 in group_id
+// already tried parallel, practically no gain
+// [[Rcpp::export]]
+std::vector<int> unequal_categories_C(const IntegerVector& group_id, const IntegerVector& item_id, const IntegerVector& item_score, const int nit, const int max_score)
+{	
+	std::vector<int> scratch((nit+1)*(max_score+1),0);
+	
+	std::vector<int> out;
+	out.reserve(nit);
+	
+	const int nr = item_id.length();	
+	const int isz = max_score+1;
+	
+	for(int i=0;i<nr;i++)
+	{
+		if(scratch[item_id[i] * isz + item_score[i]] == group_id[i])
+			scratch[item_id[i] * isz + item_score[i]] = 3;
+		else if(scratch[item_id[i] * isz + item_score[i]] == 0)
+			scratch[item_id[i] * isz + item_score[i]] = 3 - group_id[i]; //2->1, 1->2
+	}	
+	
+	for(int i=1; i<=nit; i++)
+		for(int s=0; s<=max_score; s++)
+		{
+			if(scratch[i*isz+s] == 1 || scratch[i*isz+s] == 2)
+			{
+				out.push_back(i);
+				break;
+			}
+		}
+	out.shrink_to_fit();
+	return out;	
+}
 
 
 /* ******************************************************************************************* 
@@ -989,7 +1024,7 @@ List suf_stats_nrm(const IntegerVector& booklet_id, const IntegerVector& booklet
 		plt_booklet_score[i] = std::get<1>(k);
 		plt_item_id[i] = std::get<2>(k);
 		plt_n[i] = std::get<1>(s);
-		plt_mean[i++] = (double)(std::get<0>(s))/std::get<1>(s);		
+		plt_mean[i++] = ((double)(std::get<0>(s)))/std::get<1>(s);		
 	}
 	
 	plt_booklet_id.attr("levels") = booklet_id.attr("levels");	
